@@ -118,6 +118,17 @@ def sync_blockheight(bitcoind, nodes):
         wait_for(lambda: n.rpc.getinfo()['blockheight'] == height)
 
 
+def mine_funding_to_announce(bitcoind, nodes, num_blocks=5, wait_for_mempool=0):
+    """Mine blocks so a channel can be announced (5, if it's already
+mined), but make sure we don't leave nodes behind who will reject the
+announcement.  Not needed if there are only two nodes.
+
+    """
+    bitcoind.generate_block(num_blocks - 1, wait_for_mempool)
+    sync_blockheight(bitcoind, nodes)
+    bitcoind.generate_block(1)
+
+
 def wait_channel_quiescent(n1, n2):
     wait_for(lambda: only_one(only_one(n1.rpc.listpeers(n2.info['id'])['peers'])['channels'])['htlcs'] == [])
     wait_for(lambda: only_one(only_one(n2.rpc.listpeers(n1.info['id'])['peers'])['channels'])['htlcs'] == [])
@@ -266,9 +277,12 @@ class TailableProc(object):
     def wait_for_logs(self, regexs, timeout=TIMEOUT):
         """Look for `regexs` in the logs.
 
-        We tail the stdout of the process and look for each regex in `regexs`,
-        starting from last of the previous waited-for log entries (if any).  We
-        fail if the timeout is exceeded or if the underlying process
+        The logs contain tailed stdout of the process. We look for each regex
+        in `regexs`, starting from `logsearch_start` which normally is the
+        position of the last found entry of a previous wait-for logs call.
+        The ordering inside `regexs` doesn't matter.
+
+        We fail if the timeout is exceeded or if the underlying process
         exits before all the `regexs` were found.
 
         If timeout is None, no time-out is applied.
@@ -411,7 +425,7 @@ class BitcoinD(TailableProc):
     # int > 0 := wait for at least N transactions
     # 'tx_id' := wait for one transaction id given as a string
     # ['tx_id1', 'tx_id2'] := wait until all of the specified transaction IDs
-    def generate_block(self, numblocks=1, wait_for_mempool=0):
+    def generate_block(self, numblocks=1, wait_for_mempool=0, to_addr=None):
         if wait_for_mempool:
             if isinstance(wait_for_mempool, str):
                 wait_for_mempool = [wait_for_mempool]
@@ -428,7 +442,9 @@ class BitcoinD(TailableProc):
         ))
 
         # As of 0.16, generate() is removed; use generatetoaddress.
-        return self.rpc.generatetoaddress(numblocks, self.rpc.getnewaddress())
+        if to_addr is None:
+            to_addr = self.rpc.getnewaddress()
+        return self.rpc.generatetoaddress(numblocks, to_addr)
 
     def simple_reorg(self, height, shift=0):
         """
@@ -640,6 +656,7 @@ class LightningNode(object):
                  allow_bad_gossip=False,
                  db=None, port=None, disconnect=None, random_hsm=None, options=None,
                  jsonschemas={},
+                 valgrind_plugins=True,
                  **kwargs):
         self.bitcoin = bitcoind
         self.executor = executor
@@ -675,6 +692,7 @@ class LightningNode(object):
                 self.daemon.opts["dev-debugger"] = os.getenv("DEBUG_SUBD")
             if valgrind:
                 self.daemon.env["LIGHTNINGD_DEV_NO_BACKTRACE"] = "1"
+                self.daemon.opts["dev-no-plugin-checksum"] = None
             else:
                 # Under valgrind, scanning can access uninitialized mem.
                 self.daemon.env["LIGHTNINGD_DEV_MEMLEAK"] = "1"
@@ -689,11 +707,14 @@ class LightningNode(object):
         if dsn is not None:
             self.daemon.opts['wallet'] = dsn
         if valgrind:
+            trace_skip_pattern = '*python*,*bitcoin-cli*,*elements-cli*'
+            if not valgrind_plugins:
+                trace_skip_pattern += ',*plugins*'
             self.daemon.cmd_prefix = [
                 'valgrind',
                 '-q',
                 '--trace-children=yes',
-                '--trace-children-skip=*python*,*bitcoin-cli*,*elements-cli*',
+                '--trace-children-skip={}'.format(trace_skip_pattern),
                 '--error-exitcode=7',
                 '--log-file={}/valgrind-errors.%p'.format(self.daemon.lightning_dir)
             ]
@@ -713,21 +734,16 @@ class LightningNode(object):
         if connect and not self.is_connected(remote_node):
             self.connect(remote_node)
 
-        fundingtx = self.rpc.fundchannel(remote_node.info['id'], capacity)
-
-        # Wait for the funding transaction to be in bitcoind's mempool
-        wait_for(lambda: fundingtx['txid'] in self.bitcoin.rpc.getrawmempool())
+        res = self.rpc.fundchannel(remote_node.info['id'], capacity)
 
         if confirm or wait_for_announce:
-            self.bitcoin.generate_block(1)
+            self.bitcoin.generate_block(1, wait_for_mempool=res['txid'])
 
         if wait_for_announce:
             self.bitcoin.generate_block(5)
+            wait_for(lambda: ['alias' in e for e in self.rpc.listnodes(remote_node.info['id'])['nodes']])
 
-        if confirm or wait_for_announce:
-            self.daemon.wait_for_log(
-                r'Funding tx {} depth'.format(fundingtx['txid']))
-        return {'address': addr, 'wallettxid': wallettxid, 'fundingtx': fundingtx}
+        return {'address': addr, 'wallettxid': wallettxid, 'fundingtx': res['tx']}
 
     def fundwallet(self, sats, addrtype="p2sh-segwit", mine_block=True):
         addr = self.rpc.newaddr(addrtype)[addrtype]
@@ -763,22 +779,15 @@ class LightningNode(object):
 
         self.rpc.connect(remote_node.info['id'], 'localhost', remote_node.port)
 
-        # Make sure the fundchannel is confirmed.
-        num_tx = len(self.bitcoin.rpc.getrawmempool())
         res = self.rpc.fundchannel(remote_node.info['id'], chan_capacity, feerate='slow', minconf=0, announce=announce, push_msat=Millisatoshi(chan_capacity * 500))
-        wait_for(lambda: len(self.bitcoin.rpc.getrawmempool()) == num_tx + 1)
-        blockid = self.bitcoin.generate_block(1)[0]
+        blockid = self.bitcoin.generate_block(1, wait_for_mempool=res['txid'])[0]
 
         # Generate the scid.
-        outnum = get_tx_p2wsh_outnum(self.bitcoin, res['tx'], total_capacity)
-        if outnum is None:
-            raise ValueError("no outnum found. capacity {} tx {}".format(total_capacity, res['tx']))
-
         for i, txid in enumerate(self.bitcoin.rpc.getblock(blockid)['tx']):
             if txid == res['txid']:
                 txnum = i
 
-        return '{}x{}x{}'.format(self.bitcoin.rpc.getblockcount(), txnum, outnum)
+        return '{}x{}x{}'.format(self.bitcoin.rpc.getblockcount(), txnum, res['outnum'])
 
     def getactivechannels(self):
         return [c for c in self.rpc.listchannels()['channels'] if c['active']]
@@ -879,8 +888,7 @@ class LightningNode(object):
         res = self.rpc.fundchannel(l2.info['id'], amount,
                                    announce=announce_channel,
                                    **kwargs)
-        wait_for(lambda: res['txid'] in self.bitcoin.rpc.getrawmempool())
-        blockid = self.bitcoin.generate_block(1)[0]
+        blockid = self.bitcoin.generate_block(1, wait_for_mempool=res['txid'])[0]
 
         for i, txid in enumerate(self.bitcoin.rpc.getblock(blockid)['tx']):
             if txid == res['txid']:
@@ -961,7 +969,7 @@ class LightningNode(object):
                                   + ['Received channel_update for channel {}/1'.format(c)
                                      for c in scids])
 
-    def wait_for_route(self, destination, timeout=30):
+    def wait_for_route(self, destination, timeout=TIMEOUT):
         """ Wait for a route to the destination to become available.
         """
         start_time = time.time()
@@ -1016,6 +1024,9 @@ class LightningNode(object):
         result = self.rpc.waitsendpay(rhash)
         assert(result.get('status') == 'complete')
 
+        # Make sure they're all settled, in case we quickly mine blocks!
+        dst.wait_for_htlcs()
+
     # This helper sends all money to a peer until even 1 msat can't get through.
     def drain(self, peer):
         total = 0
@@ -1151,7 +1162,7 @@ def flock(directory: Path):
     # So now we have a position in the lock, let's check if we are the
     # next one to go:
     while True:
-        files = sorted([f.resolve() for f in d.iterdir() if f.is_file()])
+        files = sorted([f for f in d.iterdir() if f.is_file()])
         # We're queued, so it should at least have us.
         assert len(files) >= 1
         if files[0] == fname:
@@ -1279,12 +1290,20 @@ class NodeFactory(object):
 
         assert len(opts) == num_nodes
 
+        # Only trace one random node's plugins, to avoid OOM.
+        if SLOW_MACHINE:
+            valgrind_plugins = [False] * num_nodes
+            valgrind_plugins[random.randint(0, num_nodes - 1)] = True
+        else:
+            valgrind_plugins = [True] * num_nodes
+
         jobs = []
         for i in range(num_nodes):
             node_opts, cli_opts = self.split_options(opts[i])
             jobs.append(self.executor.submit(
                 self.get_node, options=cli_opts,
-                node_id=self.get_node_id(), **node_opts
+                node_id=self.get_node_id(), **node_opts,
+                valgrind_plugins=valgrind_plugins[i]
             ))
 
         return [j.result() for j in jobs]
@@ -1339,7 +1358,7 @@ class NodeFactory(object):
         return node
 
     def join_nodes(self, nodes, fundchannel=True, fundamount=FUNDAMOUNT, wait_for_announce=False, announce_channels=True) -> None:
-        """Given nodes, connect them in a line, optionally funding a channel"""
+        """Given nodes, connect them in a line, optionally funding a channel, wait_for_announce waits for channel and node announcements"""
         assert not (wait_for_announce and not announce_channels), "You've asked to wait for an announcement that's not coming. (wait_for_announce=True,announce_channels=False)"
         connections = [(nodes[i], nodes[i + 1]) for i in range(len(nodes) - 1)]
 
@@ -1365,10 +1384,8 @@ class NodeFactory(object):
         for src, dst in connections:
             txids.append(src.rpc.fundchannel(dst.info['id'], fundamount, announce=announce_channels)['txid'])
 
-        wait_for(lambda: set(txids).issubset(set(bitcoind.rpc.getrawmempool())))
-
         # Confirm all channels and wait for them to become usable
-        bitcoind.generate_block(1)
+        bitcoind.generate_block(1, wait_for_mempool=txids)
         scids = []
         for src, dst in connections:
             wait_for(lambda: src.channel_state(dst) == 'CHANNELD_NORMAL')
